/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.versand.ftpclient;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.NoRouteToHostException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;

import de.bsvrz.kex.lmstic3.vew.LMSFTPListener;
import de.bsvrz.kex.lmstic3.vew.LMSKommunikationsstatus;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.LMSProperties;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Modul FTP-Client.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$
 */
public class LMSFTPClient extends Thread {

	/**
	 * DebugLogger für Debug-Ausgaben.
	 */
	private final Debug debug = Debug.getLogger();

	/**
	 * Die aktuellen Parameter.
	 */
	private LMSParameter parameter;

	/**
	 * Liste der angemeldeten Listener.
	 */
	private final List<LMSFTPListener> listenerList = new ArrayList<>();

	/**
	 * Der Kommunikationsstatus mit der Landesmeldestelle.
	 */
	private LMSKommunikationsstatus kommunikationsStatus = LMSKommunikationsstatus.NICHT_GESTOERT;

	/**
	 * Zeitpunkt des letzten Dateitransfers.
	 */
	private long lastSend;

	/**
	 * die zu versendende Datei.
	 */
	private File fileToSend;

	/**
	 * Run-Flag.
	 */
	private boolean run = true;

	/**
	 * Erzeugt eine neue Instanz der Klasse.
	 *
	 * @param parameter
	 *            Parameter
	 * @param autorun
	 *            Flag ob sich der Thread selbst starten soll
	 */
	public LMSFTPClient(final LMSParameter parameter, final boolean autorun) {
		this.parameter = parameter;
		this.setName(this.getClass().getSimpleName());

		// selbst starten
		if (autorun) {
			start();
		}
	}

	/**
	 * F&uuml;gt einen Listener hinzu.
	 *
	 * @param listner
	 *            Listener
	 */
	public void addListener(final LMSFTPListener listner) {
		listenerList.add(listner);
	}

	/**
	 * Informiert die Listener &uuml;ber versendete Dateien.
	 *
	 * @param dateien
	 *            Liste der versendeten Dateien
	 * @param status
	 *            {@link LMSKommunikationsstatus}
	 */
	void informiereListenerDateien(final List<File> dateien, final LMSKommunikationsstatus status) {
		for (final LMSFTPListener listener : listenerList) {
			listener.versendet(dateien, status);
		}
	}

	/**
	 * Informiert die Listener &uuml;ber den Kommunikationsstatus.
	 */
	void informiereListenerStatus() {
		for (final LMSFTPListener listener : listenerList) {
			listener.kommunikationsStatus(kommunikationsStatus);
		}
	}

	@Override
	public void run() {
		while (run) {
			try {
				if (fileToSend != null) {
					synchronized (fileToSend) {
						sendfile(fileToSend);
						fileToSend = null;
					}
				}

				if ((System.currentTimeMillis() - lastSend) >= parameter.getTestIntervallKommunikation()) {
					sendeKeepAlive();
				}

				Thread.sleep(parameter.getTestIntervallKommunikation());
			} catch (final InterruptedException ie) {
				/** */
			}
		}
	}

	/**
	 * Versendet eine KeepAlive Datei.
	 *
	 * @return true, wenn erfolgreich versendet, sonst false
	 */
	public boolean sendeKeepAlive() {
		debug.info("Sende KeepAlive-Datei");
		final File keepalivedatei = new File(LMSProperties.KEEP_ALIVE_DATEINAME);
		if (!keepalivedatei.exists()) {
			try {
				keepalivedatei.createNewFile();
			} catch (final IOException e) {
				debug.error(e.getMessage());
				lastSend = System.currentTimeMillis();
				return false;
			}
		}

		lastSend = System.currentTimeMillis();
		return sendfile(keepalivedatei);
	}

	/**
	 * Versendet eine Datei an die Landesmeldestelle.
	 *
	 * @param datei
	 *            zu versendende Datei
	 *
	 * @return true, wenn erfolgreich versendet, sonst false
	 */
	private boolean sendfile(final File datei) {
		boolean ok = false;
		final FTPClient ftpClient = new FTPClient();

		debug.fine("Datei: " + datei + " wird versendet");

		try {
			ftpClient.connect(parameter.getLMSHostRechner());
			// After connection attempt, you should check the reply code to
			// verify success.
			final int reply = ftpClient.getReplyCode();

			if (FTPReply.isPositiveCompletion(reply)) {
				debug.finest("verbunden");

				if (ftpClient.login(parameter.getLMSHostNutzer(), parameter.getLMSHostPasswort())) {

					debug.finest("logged in");

					boolean cont = true;
					// ftpClient.binary(); // set to binary mode transfer
					final String zielverzeichnis = parameter.getAusgangsverzeichnis();
					if ((zielverzeichnis != null) && (zielverzeichnis.length() > 0)) {
						debug.finest("wechsle Verzeichnis nach: " + zielverzeichnis);
						if (!ftpClient.changeWorkingDirectory(zielverzeichnis)) {
							cont = false;
						}
					}

					if (cont) {
						// setze binären Mode
						ftpClient.setFileType(FTP.BINARY_FILE_TYPE);

						// setze passiven Mode
						ftpClient.enterLocalPassiveMode();

						final FileInputStream in = new FileInputStream(datei.getName());
						debug.finest("starte Datei-Upload");
						final boolean result = ftpClient.storeFile(datei.getName(), in);
						debug.finest("Datei-Upload beendet");
						in.close();
						debug.finest("Verbindung geschlossen");

						ftpClient.disconnect();

						final List<File> dateiliste = new ArrayList<>();
						dateiliste.add(datei);

						if (result) {
							debug.info("Versand der Datei: " + datei + " war erfolgreich");
							informiereListenerDateien(dateiliste, LMSKommunikationsstatus.NICHT_GESTOERT);
							ok = true;
						} else {
							debug.info("Versand der Datei: " + datei + " war nicht erfolgreich");
							ok = false;
							informiereListenerDateien(dateiliste, LMSKommunikationsstatus.GESTOERT);
						}
					} else {
						ftpClient.disconnect();
						debug.error("Fehler Verzeichniswechsel zu '" + zielverzeichnis + "' auf FTP-Server ");
						ok = false;
					}
				} else {
					ftpClient.disconnect();
					debug.error("LOGIN-Fehler");
					ok = false;
				}
			} else {
				ftpClient.disconnect();
				debug.error("Verbindung zurückgesetzt");
				ok = false;
			}
		} catch (final IOException exception) {
			if (exception instanceof UnknownHostException) {
				debug.error("Der Zielrechner '" + parameter.getLMSHostRechner() + "' ist unbekannt");
			} else if (exception instanceof NoRouteToHostException) {
				debug.error(exception.getLocalizedMessage());
				// debug.error("Keine Route zum Zielrechner");
			} else {
				debug.error(exception.getClass().getCanonicalName() + ": " + exception.getMessage());
			}

			ok = false;
		}

		if (ok) {
			kommunikationsStatus = LMSKommunikationsstatus.NICHT_GESTOERT;
		} else {
			kommunikationsStatus = LMSKommunikationsstatus.GESTOERT;
		}

		informiereListenerStatus();
		lastSend = System.currentTimeMillis();

		return ok;
	}

	/**
	 * Setzt die zu versendende Datei.
	 *
	 * @param datei
	 *            Datei zum Versand
	 */
	public void setVersandDatei(final File datei) {
		fileToSend = datei;
	}

	/**
	 * Stoppt den Thread.
	 */
	public void stopp() {
		this.run = false;
		this.interrupt();
	}

}
