/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.kex.lmstic3.empfang.erzeugung.Meldungserzeugung;
import de.bsvrz.kex.lmstic3.versand.kodierer.Meldungskodierer;
import de.bsvrz.kex.lmstic3.vew.daten.DavDatenModell;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Hauptklasse f&uuml;r die SWE 2 LMS-TIC3.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$
 */
public class LMSTIC3 implements StandardApplication {

	/**
	 * Landesmeldestellen-Objekt.
	 */
	private SystemObject lmsobjekt;

	private static final String LMS_TYP = "typ.tic3LandesMeldeStelle";

	/** Die Menge der TMC-Meldungen. */
	public static final String MENGE_TMC_MELDUNGEN = "TMCMeldungen";
	/** Der Typ der TMC-Meldungen. */
	public static final String TYP_TMC_MELDUNGEN = "typ.tmcVerkehrsMeldung";

	/** Die Menge der TIC3-Meldungen. */
	public static final String MENGE_TIC3_MELDUNGEN = "TMCTIC3Meldungen";
	/** Der Typ der TIC3-Meldungen. */
	public static final String TYP_TIC3_MELDUNGEN = "typ.tmcTic3infoMeldung";

	/**
	 * Worker-Thread.
	 */
	private class KexLMSWorker extends Thread {

		@Override
		public void run() {
			Meldungskodierer mk = null;
			Meldungserzeugung md = null;
			String fehler = null;

			LMSTIC3LogTools.log(debug, LmsMsg.InfoApplikationStartet);

			DavDatenModell.getInstanz().setVerbindung(dav);

			try {
				// Komponente Meldungsversand starten
				mk = new Meldungskodierer(dav, (ConfigurationObject) lmsobjekt, mitValidierung);

				// ohne diese Pause geht der Meldungsdekodierer manchmal in
				// Fehler: "Die Menge 'TMCMeldungen' existiert nicht am
				// Objekt..."
				Thread.sleep(5000);

				// Komponente Meldungsempfang starten
				md = new Meldungserzeugung(dav, (ConfigurationObject) lmsobjekt, mitValidierung);

				while (true) {
					Thread.sleep(10000);
					if (!mk.isAlive()) {
						// LogTools.log(debug, LmsMsg.ErrorThreadBeendet,
						// "Meldungskodierer");
						md.stopp();
						fehler = "Meldungsversand-Thread beendet";
						break;
					}

					if (!md.isAlive()) {
						// LogTools.log(debug, LmsMsg.ErrorThreadBeendet,
						// "Meldungsdekodierer");
						fehler = "Meldungsempfang-Thread beendet";
						mk.stopp();
						break;
					}
				}
			} catch (final Exception e) {
				debug.error(e.getMessage());
			}

			if ((mk != null) && mk.isAlive()) {
				mk.stopp();
				try {
					mk.join();
				} catch (final InterruptedException e) {
				}
			}

			if ((md != null) && md.isAlive()) {
				md.stopp();
				try {
					mk.join();
				} catch (final InterruptedException e) {
				}
			}

			if (fehler != null) {
				LMSTIC3LogTools.log(debug, LmsMsg.ErrorAppBeendet, fehler);
				dav.disconnect(true, fehler);
				System.exit(-1);
			} else {
				LMSTIC3LogTools.log(debug, LmsMsg.InfoApplikationBeendet);
			}
		}
	}

	/**
	 * MAIN-Methode.
	 *
	 * @param args
	 *            Argumente
	 */
	public static void main(final String[] args) {
		Thread.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {

			@Override
			public void uncaughtException(final Thread t, final Throwable e) {
				Logger.getLogger(getClass().getName()).log(Level.INFO, "*** Application wird beendet ***", e); //$NON-NLS-1$
				dav.disconnect(true, e.getMessage());
				System.exit(-1);
			}

		});

		StandardApplicationRunner.run(new LMSTIC3(), args);
	}

	/**
	 * Logger für Debug-Ausgaben.
	 */
	private Debug debug;

	/**
	 * Die Verbindung zum Datenverteiler.
	 */
	private static ClientDavConnection dav;

	private boolean mitValidierung;

	private String lmsPid;

	/**
	 * Standard-Konstruktor.
	 */
	protected LMSTIC3() {
		super();
	}

	@Override
	public void initialize(final ClientDavInterface connection) throws Exception {
		debug = Debug.getLogger();
		dav = (ClientDavConnection) connection;

		try {
			initLms(dav);

			new KexLMSWorker().start();
		} catch (final Exception e) {
			LMSTIC3LogTools.log(debug, LmsMsg.ErrorAppException, e.getMessage());
			throw (e);
		}
	}

	/**
	 * <p>
	 * PID oder ID der Landesmeldestelle (vom Typ ‚typ.tic3LandesMeldeStelle’
	 * mit der Meldungen ausgetauscht werden sollen.
	 * </p>
	 * <p>
	 * Wenn das Argument nicht angegeben wird, darf nur ein Objekt dieses Typs
	 * in der Konfiguration vorhanden sein, das dann von der SWE benutzt wird.
	 * </p>
	 * 
	 * @param connection
	 *            {@link ClientDavConnection}
	 * @throws TIC3Exception
	 *             bei Fehlern
	 */
	private void initLms(final ClientDavConnection connection) throws TIC3Exception {

		if (lmsPid != null && lmsPid.length() > 0) {
			// teste, ob das angegebene Landesmeldestellen-Objekt existiert
			lmsobjekt = dav.getDataModel().getObject(lmsPid);

			if (lmsobjekt == null) {
				throw new TIC3Exception(
						"Das angegebene Objekt '" + lmsPid + "' kann nicht vom Datenverteiler gelesen werden!");
			}

			if (!lmsobjekt.isOfType(LMS_TYP)) {
				throw new TIC3Exception("Das angegebene Objekt '" + lmsPid + "' ist nicht vom Typ '" + LMS_TYP + "'!");
			}
		} else {
			final SystemObjectType type = dav.getDataModel().getType(LMS_TYP);
			if (type == null) {
				throw new TIC3Exception(
						"Das ObjektTyp '" + LMS_TYP + "' kann nicht vom Datenverteiler gelesen werden!");
			}

			final List<SystemObject> elements = type.getElements();

			if (elements.size() == 0) {
				throw new TIC3Exception(
						"Es exitieren keine Objekte vom ObjektTyp '" + LMS_TYP + "' in der Konfiguration!");
			}

			if (elements.size() > 1) {
				throw new TIC3Exception(
						"Es exitiert mehr als ein Objekt vom ObjektTyp '" + LMS_TYP + "' in der Konfiguration!");
			}

			lmsobjekt = elements.get(0);

			LMSTIC3LogTools.log(debug, LmsMsg.InfoBenutzeLMS, lmsobjekt.getName());
		}
	}

	/**
	 * Wertet die an die Applikation &uuml;bergebenen Kommandozeilenparameter
	 * aus.<br>
	 * Zus&auml;tzlich zu den Standard-Datenverteiler-Argumenten sind folgende
	 * Argumente m&ouml;glich:
	 * <ul>
	 * <li>-LMS=&lt;Konfigurationsobjekt&gt;, PID oder ID der Landesmeldestelle,
	 * mit der Meldungen ausgetauscht werden sollen</li>
	 * </ul>
	 *
	 * @param argumentList
	 *            die Liste der &uuml;bergebenen Argumente
	 * @throws Exception
	 *             bei Ausnahmen
	 * @see de.bsvrz.sys.funclib.application.StandardApplication#parseArguments(de.bsvrz.sys.funclib.commandLineArgs.ArgumentList)
	 */
	@Override
	public void parseArguments(final ArgumentList argumentList) throws Exception {
		mitValidierung = argumentList.fetchArgument("-pruefeSchema" + "=nein").booleanValue();
		lmsPid = argumentList.fetchArgument("-LMS" + "=").asString();
	}
}
