/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.AbstractTIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TMCInteger;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3TmcEventCodeTyp.TIC3TmcEventCodeTypEN;

/**
 * TMC-codierte Ereignisse.
 * <p>
 * Entspricht TIC3-Typ 'TmcEventAndQuantifierType'.
 * </p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$
 */
public class TIC3TMCEvent extends AbstractTIC3InfoElement {

	/**
	 * TIC3 Info XML Element: TmcEvent (TMC), A list of one or more TMC event
	 * codes with additional information which are equivalent to "TIC3 event"
	 * codes, where TMC is defined in the TMC standard ISO 14819-2.
	 */
	public static final String XML_ELEMENT_TMCEVENT = "TmcEvent";

	/**
	 * DAV-Attribut.
	 */
	public static final String DAV_ATTR_TMCEVENT = "TIC3TmcEvent";

	/**
	 * TMC-Code.
	 * <p>
	 * TIC3 Info XML Element: Code (COD), The TMC event or supplementary
	 * information code.
	 * </p>
	 */
	private final TMCInteger tmcCode = new TMCInteger("Code", "Code");

	/**
	 * TMC-Eventcode Typ.
	 * <p>
	 * TIC3 Info XML Element: Type (TYP), The type of TMC event code such as
	 * "Event code" or "Supplementary information code".
	 * </p>
	 */
	private TIC3TmcEventCodeTyp tmcEventCodeType = new TIC3TmcEventCodeTyp();

	/**
	 * Update-Klasse.
	 * <p>
	 * TIC3 Info XML Element: UpdateClass (UCL), The update class to which the
	 * TMC event code belongs.
	 * </p>
	 */
	private final TMCInteger updateClass = new TMCInteger("UpdateClass", "UpdateClass");

	/**
	 * Quantifizierer.
	 * <p>
	 * TIC3 Info XML Element: Quantifier (QUA), A list of additional quantifiers
	 * used in combination with the TMC event code.
	 * </p>
	 */
	private List<TIC3TmcQuantifier> quantifier = new ArrayList<>();

	/**
	 * Erzeugt ein neues Objekt.
	 */
	public TIC3TMCEvent() {
		super(XML_ELEMENT_TMCEVENT, null);
		addKindElement(tmcCode);
		addKindElement(tmcEventCodeType);
		addKindElement(updateClass);
	}

	/**
	 * Erzeugt ein neues Objekt.
	 * 
	 * @param typ
	 *            {@link TIC3TmcEventCodeTypEN}
	 * @param code
	 *            Event-Code
	 */
	public TIC3TMCEvent(final TIC3TmcEventCodeTypEN typ, final int code) {
		this();
		tmcCode.setWert(code);
		tmcEventCodeType.setEventTyp(typ);
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		for (final TIC3InfoElement child : getKindElemente()) {
			child.publizieren(davdata, parameter);
		}

		if (quantifier.size() > 0) {
			final Data.Array array = davdata.getArray(TIC3TmcQuantifier.DAV_ATTR_QUANTIFIER);
			array.setLength(quantifier.size());
			for (int i = 0; i < quantifier.size(); i++) {
				quantifier.get(i).publizieren(array.getItem(i), parameter);
			}
		}
	}

	@Override
	public void einlesenXML(final Node node) throws TIC3Exception {

		super.einlesenXML(node);
		final NodeList nl = node.getChildNodes();

		for (int i = 0; i < nl.getLength(); i++) {
			final Node n = nl.item(i);
			if (n.getNodeName().equals(TIC3TmcQuantifier.XML_ELEMENT_QUANTIFIER)) {
				final TIC3TmcQuantifier q = new TIC3TmcQuantifier();
				q.einlesenXML(n);
				quantifier.add(q);
			}
		}
	}

	@Override
	public String toString() {
		return this.getClass().getSimpleName();
	}

	public TIC3TmcEventCodeTypEN getEventTyp() {
		return tmcEventCodeType.getEventTyp();
	}

	public void setEventTyp(final TIC3TmcEventCodeTypEN typ) {
		tmcEventCodeType.setEventTyp(typ);
	}

	public int getCode() {
		return tmcCode.getWert();
	}
}
